﻿using System;
using System.Collections.Generic;
using System.Windows.Controls;
using Microsoft.Practices.Prism.Commands;
using Microsoft.Practices.Prism.Events;
using Microsoft.Practices.Unity;
using MsdnDemo.Events;
using MsdnDemo.Infrastructure.Portable.Events;
using MsdnDemo.Infrastructure.Portable.Interfaces;
using MsdnDemo.Infrastructure.Portable.Logger;
using MsdnDemo.Infrastructure.Portable.MvpVmBase;
using MsdnDemo.Infrastructure.Portable.Types;
using MsdnDemo.Phone.Events;

namespace MsdnDemo.Base
{
    public class PresentationPresenterBase<TView,TViewModel>
        : MvpVmPresenter<TView, TViewModel>
            where TView : IView
            where TViewModel : IMvpVmViewModel
    {
        #region Fields 
        private ILogger _logger;
        private IEventAggregator _eventAggregator;
        private readonly List<ViewEventArgs> _queuedViewEvents = new List<ViewEventArgs>();
        #endregion

        [Dependency]
        public ILogger Logger
        {
            get { return _logger; }
            set { 
                _logger = value;
                TryInitialize();
            }
        }
        
        [Dependency]
        public new IShell Shell
        {
            get { return base.Shell; }
            set
            {
                base.Shell = value;
                TryInitialize();
            }
        }

        [Dependency]
        public new TView View
        {
            get { return base.View; }
            set
            {
                base.View = value;

                ViewPropertySet();

                TryInitialize();
            }
        }

        [Dependency]
        public IEventAggregator EventAggregator
        {
            get { return _eventAggregator; }
            set
            {
                _eventAggregator = value;
                
                // If view events were fired while the EventAggregator
                // was not yet instantiated they will be queued.  We'll
                // need to publish them
                foreach (var viewEvent in _queuedViewEvents)
                    EventAggregator.GetEvent<ViewEvent>()
                        .Publish(viewEvent);
                
                _queuedViewEvents.Clear();

                // Subscribe to message events
                EventAggregator.GetEvent<MessageEvent>()
                    .Subscribe(MessageEventHandler);

                SubscribeToDataEvent(HandleDataArrivedEvent);
            }
        }

        [Dependency]
        public new TViewModel ViewModel
        {
            get { return base.ViewModel; }
            set
            {
                base.ViewModel = value;
                var viewModel = value as PresentationViewModelBase;

                // Wire up our button command which the ButtonStyle
                // has a Command binding to
                viewModel.ButtonCommand = new DelegateCommand<object>(
                    ExecuteButtonCommandHandler,
                    CanExecuteButtonCommandHandler);

                viewModel.Presenter = this;

                TryInitialize();
            }
        }

        private void TryInitialize()
        {
            if (View == null || ViewModel == null || Shell == null|| Logger == null)
                return;

            View.DataContext = ViewModel;

            // Hook for subclass so that it can initializes 
            // with all dependency properties set
            Initialize();
        }

        protected virtual void Initialize()
        {
            Logger.Log("{0}.Initialize()", GetType());
        }


        private bool CanExecuteButtonCommandHandler(object e)
        {
            var args = new DelegateCommandEventArgs{Sender = e};
            return CanExecuteButtonCommandHandler(this, args);
        }

        protected virtual bool CanExecuteButtonCommandHandler(object sender, DelegateCommandEventArgs e)
        {
            return true;
        }

        private void ExecuteButtonCommandHandler(object e)
        {
            DelegateCommandEventArgs args;
            if (e is DelegateCommandEventArgs)
                args = e as DelegateCommandEventArgs;
            else
                args = new DelegateCommandEventArgs{Sender = e};

            // Give virtual method an opportunity to validate
            Logger.Log("{0}", args);

            if (CanExecuteButtonCommandHandler(this, args))
                ExecuteButtonCommandHandler(this, args);
        }

        protected virtual void ExecuteButtonCommandHandler(object sender, DelegateCommandEventArgs e)
        {
            string message = null;
            if (e.ViewModel == null)
            {
                var button = e.Sender as Button;
                message = string.Format("{0}", e.Sender);
            }
            else
            {
                message = string.Format(
                    "OVERRIDE ExecuteButtonCommandHandler\r\nSender: {0}\r\nVM: {1}\r\nID: {2}",
                    e.Sender,
                    e.ViewModel.GetType().Name,
                    e.Tag.ToString());

            }
            Logger.Log(message);
        }

        public override void ViewActivated(ViewEventArgs e)
        {
            Logger.Log("{0} ViewActivated", GetType().Name);
        }

        private void ViewPropertySet()
        {
            // Event aggregator will not be available so we'll queue
            // it until EventAggregator set.
            var viewEvent = new ViewEventArgs
            {
                View = View,
                ProcessType = ProcessType.NewView
            };

            // Process or queue event
            if (EventAggregator != null)
                EventAggregator.GetEvent<ViewEvent>().Publish(viewEvent);
            else
                _queuedViewEvents.Add(viewEvent);
        }

        /// <summary>
        /// Subscribes to data event.
        /// </summary>
        /// <param name="callback">The callback.</param>
        protected void SubscribeToDataEvent(Action<DataEventArgs> callback)
        {
            EventAggregator.GetEvent<DataEvent>().Subscribe(callback);
        }


        /// <summary>
        /// Handles the data arrived event.   
        /// Has to be public method since it handles event
        /// subscription.
        /// </summary>
        /// <param name="e">The <see cref="DataEventArgs"/> 
        /// instance containing the event data.</param>
        public virtual void HandleDataArrivedEvent(DataEventArgs e)
        {
            try
            {
                switch (e.ProcessType)
                {
                    case ProcessType.DataCreate:
                    case ProcessType.DataDelete:
                    case ProcessType.DataList:
                    case ProcessType.DataRead:
                    case ProcessType.DataUpdate:
                        break;
                }
            }
            catch (Exception ex)
            {
                Logger.Log("{0}:MvpVmPresenter.HandleDataArrivedEvent({1}): \r\n\t{2}{3}",
                    GetType().Name,
                    e.ProcessType,
                    ex.Message,
                    ex.StackTrace);
            }
        }

        /// <summary>
        /// Message event handler.
        /// </summary>
        /// <param name="e">The <see cref="MsdnDemo.Infrastructure.Portable.Events.MessageEventArgs"/> instance containing the event data.</param>
        public virtual void MessageEventHandler(MessageEventArgs e)
        {
            Logger.Log("MESSAGE EVENT:{0} IsError:{1} Status:{2} ({3})",
                       e.Message, e.IsError, e.StatusCode, GetType().Name);
        }


        protected void RaiseMessageEvent(string message)
        {
            EventAggregator.GetEvent<MessageEvent>()
                .Publish(new MessageEventArgs
                             {
                                 Message = message,
                                 Type = ProcessType.Message,
                                 Sender = this
                             });
        }

        /// <summary>
        /// Raises the view event.
        /// </summary>
        /// <param name="moduleName">Name of the module.</param>
        /// <param name="viewName">Name of the view.</param>
        /// <param name="processType">Type of the process.</param>
        protected void RaiseViewEvent(string moduleName, string viewName, ProcessType processType)
        {
            EventAggregator.GetEvent<ViewEvent>().Publish(new ViewEventArgs
            {
                ModuleName = moduleName,
                ProcessType = ProcessType.ActivateView,
                ViewNameToActivate = viewName
            });
        }

        /// <summary>
        /// Notify the ApplicationController that the login was successful so that it can
        /// load (on demand) the password manager module (as configured in the Bootstrapper)
        /// Once modules are loaded they will not be loaded again so it is harmless on
        /// successive calls.
        /// </summary>
        /// <param name="moduleName">Name of the module.</param>
        /// <param name="viewName">Name of the view.</param>
        /// <param name="eventType">Type of the event.</param>
        protected void RaiseModuleEvent(string moduleName, string viewName, ModuleEventType eventType)
        {
            EventAggregator.GetEvent<ModuleEvent>()
                .Publish(new ModuleEventArgs
                {
                    ModuleName = moduleName,
                    EventType = eventType,
                    PayLoad = viewName // The view to load on click
                });            
        }
    }
}
