﻿using System;
using Microsoft.Practices.Prism.Events;
using Microsoft.Practices.Unity;
using MsdnDemo.Events;
using MsdnDemo.Infrastructure.Portable.Constants;
using MsdnDemo.Infrastructure.Portable.Events;
using MsdnDemo.Infrastructure.Portable.Interfaces;
using MsdnDemo.Infrastructure.Portable.Logger;
using MsdnDemo.Infrastructure.Portable.Types;

namespace MsdnDemo.LayerBusinessLogic.Base
{
    public class BllBase<TEntity> : IBll
    {
        public bool IsFailRequiredForUnitTest { get; set; }

        [Dependency]
        public IDal<TEntity> Dal { get; set; }

        [Dependency]
        public TEntity Entity { get; set; }

        [Dependency]
        public IEventAggregator EventAggregator { get; set; }

        [Dependency]
        public ILogger Logger { get; set; }

        public virtual void Initialize(){}
     
        
        /// <summary>
        /// Creates the table.
        /// </summary>
        /// <returns></returns>
        public int CreateTable()
        {
            try
            {
                var result = Dal.CreateTable(this, null);

                if (IsFailRequiredForUnitTest)
                    throw new Exception(Messages.UnitTestBogusErrorMessage);

                return result;
            }
            catch (Exception ex)
            {
                // See if the error was because the table already exists
                var statusCode = StatusCode.NotDefined;
                if (ex.Message.ToLower().Contains(Messages.AlreadyExists))
                    statusCode = StatusCode.AlreadyExists;

                // Publish message event
                EventAggregator.GetEvent<MessageEvent>()
                    .Publish(new MessageEventArgs
                    {
                        Exception = ex,
                        Message = string.Format(Messages.CouldNotCreateTableP0, typeof(TEntity).Name),
                        Type = ProcessType.CreateTable,
                        ErrorCode = StatusCode.TableCreateError,
                        StatusCode = statusCode
                    });

                return StatusCode.TableCreateError;
            }
        }

        /// <summary>
        /// Drops the table.
        /// </summary>
        /// <returns></returns>
        public int DropTable()
        {
            try
            {
                var result = Dal.DropTable(this, null);

                if (IsFailRequiredForUnitTest)
                    throw new Exception(Messages.UnitTestBogusErrorMessage);

                return result;
            }
            catch (Exception ex)
            {
                // Publish message event
                EventAggregator.GetEvent<MessageEvent>()
                    .Publish(new MessageEventArgs
                    {
                        Exception = ex,
                        Message = string.Format(Messages.CouldNotDropTableP0, typeof(TEntity).Name),
                        Type = ProcessType.CreateTable,
                        ErrorCode = StatusCode.TableDropError,
                        StatusCode = StatusCode.NotDefined
                    });

                return StatusCode.TableCreateError;
            }
        }

        /// <summary>
        /// Creates the specified record.  Expects e.Data
        /// to be populated with record to create
        /// </summary>
        /// <param name="e">The <see cref="MsdnDemo.Infrastructure.Portable.Events.DalEventArgs"/> instance containing the event data.</param>
        /// <returns></returns>
        public int Create(DalEventArgs e)
        {
            e.Type = ProcessType.DataCreate;
            var result = Create(this, e);

            if (result == StatusCode.DoesNotExist)
            {
                result = CreateTableWhenNotExist(e);
                if (result == StatusCode.Success)
                    return Create(e);
            }
            return result;
        }

        private int Create(object sender, DalEventArgs e)
        {
            e.Type = ProcessType.DataCreate;
            try
            {
                var result = Dal.Create(this, e);

                // Store the primary key returned by
                // data access layer
                e.PrimaryKey = result;

                if (e.Exception != null)
                {
                    if (e.Exception.Message.Contains("column id is not unique"))
                        return StatusCode.UniquenessConstraint;

                    return StatusCode.RecordCreateError;
                }

                return StatusCode.Success;
            }
            catch(Exception ex)
            {
                if (ex.Message.Contains("DEVELOPER"))
                    throw ex;

                if (ex.Message.Contains(Messages.NoSuchTable))
                    return StatusCode.DoesNotExist;

                // Publish message event
                EventAggregator.GetEvent<MessageEvent>()
                    .Publish(new MessageEventArgs
                    {
                        Exception = ex,
                        Message = string.Format(Messages.CouldNotCreateRecordP0, typeof(TEntity).Name),
                        Type = ProcessType.DataCreate,
                        ErrorCode = StatusCode.RecordCreateError,
                        StatusCode = StatusCode.NotDefined
                    });

                return StatusCode.TableCreateError;                
            }
        }

        /// <summary>
        /// Reads the record specified by e.PrimaryKey
        /// </summary>
        /// <param name="e">The <see cref="MsdnDemo.Infrastructure.Portable.Events.DalEventArgs"/> instance containing the event data.</param>
        /// <returns></returns>
        public int Read(DalEventArgs e)
        {
            e.Type = ProcessType.DataRead;
            e.Results = Dal.Read(this, e);

            if (e.Results == null)
                return StatusCode.NoData;

            return StatusCode.Success;
        }

        /// <summary>
        /// Updates the specified e.
        /// </summary>
        /// <param name="e">The <see cref="MsdnDemo.Infrastructure.Portable.Events.DalEventArgs"/> instance containing the event data.</param>
        /// <returns></returns>
        public int Update(DalEventArgs e)
        {
            e.Type = ProcessType.DataUpdate;
            var status = Dal.Update(this, e);

            return status;
        }

        /// <summary>
        /// Deletes the specified record
        /// </summary>
        /// <param name="e">The <see cref="MsdnDemo.Infrastructure.Portable.Events.DalEventArgs"/> instance containing the event data.</param>
        /// <returns></returns>
        public int Delete(DalEventArgs e)
        {
            e.Type = ProcessType.DataDelete;
            var status = Dal.Delete(this, e);
            
            return status;
        }

        /// <summary>
        /// Lists the specified e.
        /// </summary>
        /// <param name="e">The <see cref="MsdnDemo.Infrastructure.Portable.Events.DalEventArgs"/> instance containing the event data.</param>
        /// <returns></returns>
        public int List(DalEventArgs e)
        {
            var isError = false;
         
            tryAgain:
            try
            {
                e.Results = Dal.ReadList(this, e);
            }
            catch(Exception ex)
            {
                if (!isError && ex.Message.Contains(Messages.NoSuchTable))
                {
                    isError = true;
                    var status = CreateTableWhenNotExist(e);
                    if (status != StatusCode.Success)
                        return StatusCode.TableCreateError;
                    goto tryAgain;
                }
                throw;
            }
            // Shouldn't be null
            if (e.Results == null)
                return StatusCode.NoData;

            ListDataArrivedHandler(e);

            return StatusCode.Success;
        }

        protected virtual int CreateTableWhenNotExist(DalEventArgs e)
        {
            return CreateTable();
        }

        protected virtual void ListDataArrivedHandler(DalEventArgs e)
        {
            Logger.Log("VIRTUAL: {0}.ListDataArrived()", GetType().Name);
        }

    }
}
