﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Practices.Unity;
using MsdnDemo.Events;
using MsdnDemo.Infrastructure.Portable.Constants;
using MsdnDemo.Infrastructure.Portable.Entities;
using MsdnDemo.Infrastructure.Portable.Events;
using MsdnDemo.Infrastructure.Portable.Interfaces;
using MsdnDemo.LayerBusinessLogic.Base;
using MsdnDemo.LayerBusinessLogic.Interfaces;

namespace MsdnDemo.LayerBusinessLogic
{
    public class UserBll : BllBase<UserEntity>, IUserBll
    {
        private bool _isProcessing;

        [Dependency]
        public IRolesBll RoleBll { get; set; }

        [Dependency]
        public IUserRoleBll UserRoleBll { get; set; }

        [Dependency]
        public ISecurityViewModel SecurityViewModel { get; set; }

        public List<RoleEntity> Roles { get; set; }

        public override void Initialize()
        {
            // Get the system roles
            var roleResults = new DalEventArgs();
            var status = RoleBll.List(roleResults);
            if (status != StatusCode.Success)
                throw new Exception("Could not get system roles!", roleResults.Exception);

            // Get a list of roles supported by system.  We'll
            // use this is AuthenticateUser to join with the 
            // user roles (to get role name)
            Roles = roleResults.GetResults<IEnumerable<RoleEntity>>().ToList();
        }

        private IEnumerable<UserRoleEntity> GetUserRoles()
        {
            // Get user roles from user/role xref table
            var userRoleResults = new DalEventArgs
            {
                WhereClause = string.Format("UserFk={0}", SecurityViewModel.Id)
            };

            var status = UserRoleBll.List(userRoleResults);
            if (status != StatusCode.Success)
                throw new Exception("Could not get user roles!", userRoleResults.Exception);

            // Get the user roles from the above List() 
            var userRoles = userRoleResults.GetResults<IEnumerable<UserRoleEntity>>();

            return userRoles;
        }


        public int DropRoleFromUser(string roleList)
        {
            var status = StatusCode.Success;

            var userRoles = GetUserRoles().ToList();

            foreach(var role in roleList.Split(','))
            {
                var systemRole = Roles.First(r => r.Role == role);
                if (systemRole == null) // Invalid role
                    continue;
                
                var roleId = systemRole.Id;
                var userRoleId = userRoles.FirstOrDefault(u => u.RoleFk == roleId);
                if (userRoleId == null) // User does not have role
                    continue;

                // Prepare to delete the role from the user
                var args = new DalEventArgs {PrimaryKey = userRoleId.Id};
                status = UserRoleBll.Delete(args);
                if (status != StatusCode.Success)
                    Logger.Log("Could not delete role {0} \r\n{1}{2}", 
                        role,
                        args.Exception.Message, 
                        args.Exception.StackTrace);
            }
            
            // Get the user roles from the user role table
            SecurityViewModel.Roles =
                (from u in GetUserRoles()
                 join r in Roles on u.RoleFk equals r.Id
                 select r.Role).ToList();

            return status;
        }


        public int CreateUsersWithRoles(UserEntity userEntity, string roleList)
        {
            var roles = roleList.Split(',')
                .Select(role => new RoleEntity {Role = role}).ToList();

            return CreateUsersWithRoles(userEntity, roles);
        }

        public int CreateUsersWithRoles(UserEntity userEntity, List<RoleEntity> roleList)
        {
            var userResult = new DalEventArgs {Data = userEntity};
            var status = Create(userResult);
            if (status != StatusCode.Success)
                throw new Exception("Could not create user with roles", userResult.Exception);

            var userRecord = userResult.GetResults<UserEntity>();
            userRecord.Id = userResult.PrimaryKey;

            // Build UserRoleEntity records
            var userRoles =
                from r in Roles
                join l in roleList on r.Role equals l.Role
                select new UserRoleEntity
                           {
                                RoleFk = r.Id,
                                UserFk = userRecord.Id
                           };
                
            // Create UserRoleEntity records in table
            foreach(var role in userRoles)
                UserRoleBll.Create(new DalEventArgs {Data = role});

            return StatusCode.Success;
        }


        public bool AuthenticateUser(string password)
        {
            var returnArgs = new DalEventArgs
                {
                    // Dal is responsible for processing the where clause
                    // as applicable - we only want records where the 
                    // primary email (could be multiple accounts) is equal
                    // to the login
                    WhereClause = string.Format("PrimaryEmail='{0}'",SecurityViewModel.Login)
                };

            // Get the list of users that have a PrimaryEmail=Login
            var status = List(returnArgs);

            // If there was an exception (not failed login)
            if (status != StatusCode.Success)
                throw new Exception("Could not authenticate!", returnArgs.Exception);

            // Get the user list from the results
            var userList = returnArgs.GetResults<IEnumerable<UserEntity>>();

            // If multiple accounts we want to ensure we check all of them
            // against provided password
            var user = userList.FirstOrDefault(u => 
                   u.PrimaryEmail == SecurityViewModel.Login
                && u.Password == password );

            // Set flag
            SecurityViewModel.IsAuthenticated = user != null;

            if (SecurityViewModel.IsAuthenticated)
            {
                SecurityViewModel.Id = user.Id;

                var userRoles = GetUserRoles();

                // Get the user roles from the user role table
                SecurityViewModel.Roles =
                    (from u in userRoles
                     join r in Roles on u.RoleFk equals r.Id
                     select r.Role).ToList();

            }
            return SecurityViewModel.IsAuthenticated;
        }


        protected override int CreateTableWhenNotExist(DalEventArgs e)
        {
            var status = base.CreateTableWhenNotExist(e);
            if (status == StatusCode.Success)
                status = CreateDefaultUser(e);
            return status;
        }


        private int CreateDefaultUser(DalEventArgs e)
        {
            CreateUsersWithRoles(new UserEntity
                                     {
                                         Id = 1,
                                         FirstName = "System",
                                         LastName = "Administrator",
                                         PrimaryEmail = "admin@mydomain.net",
                                         Password = "password"
                                     }, "Admin,User");

            CreateUsersWithRoles(new UserEntity
                                     {
                                         FirstName = "Jane",
                                         LastName = "Doe",
                                         Password = "password",
                                         PrimaryEmail = "jane@mydomain.net"
                                     }, "User");

            CreateUsersWithRoles(new UserEntity
                                     {
                                         FirstName = "Guest",
                                         LastName = "User",
                                         Password = "password",
                                         PrimaryEmail = "guest@mydomain.net"
                                     }, "Guest");

            return StatusCode.Success;
        }


        public List<UserEntity> GetUserList()
        {
            var userArgs = new DalEventArgs();

            // Request the user list
            var status = List(userArgs);

            // If not successful than raise event for subscribers
            if(status!=StatusCode.Success)
            {
                EventAggregator.GetEvent<MessageEvent>()
                    .Publish(new MessageEventArgs
                                 {
                                     Exception = userArgs.Exception,
                                     Message = "Could not get User List!",
                                     ErrorCode = StatusCode.ListFailed,
                                     IsError = true
                                 });
                return new List<UserEntity>();
            }

            // Get the list from the arguments
            var users = userArgs.GetResults<List<UserEntity>>();

            EventAggregator.GetEvent<MessageEvent>()
                .Publish(new MessageEventArgs
                             {
                                 Message = string
                                 .Format("{0} user records",users.Count)
                             });
            return users;
        }
    }
}
