﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using Community.CsharpSqlite.Gwn.Library.SqliteDataProvider;
using Community.CsharpSqlite.Gwn.Library.SqliteDataProvider.Base;
using Community.CsharpSqlite.Gwn.Library.SqliteDataProvider.Exceptions;
using Microsoft.Practices.Prism.Logging;
using Microsoft.Practices.Unity;
using MsdnDemo.Infrastructure.Portable.Constants;
using MsdnDemo.Infrastructure.Portable.Events;
using MsdnDemo.Infrastructure.Portable.Interfaces;
using MsdnDemo.Infrastructure.Portable.Logger;
using MsdnDemo.Infrastructure.Portable.Types;
using MsdnDemo.LayerBusinessLogic.Interfaces;
using MsdnDemo.LayerDataAccess.Sqlite;

namespace MsdnDemo.LayerDataAccess.Sql
{
    /// <summary>
    /// Data Access Layer base class
    /// </summary>
    /// <typeparam name="TEntity">The type of the entity.</typeparam>
    public class SqlDal<TEntity> : IDal<TEntity>
    {
        [Dependency]
        public ILogger Logger { get; set; }

        [Dependency]
        public ISecurityViewModel SecurityViewModel { get; set; }

        public IUserBll UserBll { get; set; }

        /// <summary>
        /// Occurs when [data arrived event].
        /// </summary>
        public event EventHandler<DataEventArgs> DataArrivedReadEvent;

        /// <summary>
        /// Gets or sets the connection string.
        /// </summary>
        /// <value>The connection string.</value>
        public string ConnectionString { get; set; }

        /// <summary>
        /// Gets or sets a value indicating whether this instance is default data used during create.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is default data used during create; otherwise, <c>false</c>.
        /// </value>
        public bool IsDefaultDataUsedDuringCreate { get; set; }

        /// <summary>
        /// SqliteHelper reference
        /// </summary>
        public SqlUtility SqlUtility = new SqlUtility(typeof(TEntity));


        /// <summary>
        /// Initializes a new instance of the <see cref="SqlDal&lt;TEntity&gt;"/> class.
        /// </summary>
        public SqlDal() : this("MsdnDemo.1.0.0.5"){}

        /// <summary>
        /// Initializes a new instance of the <see cref="SqlDal&lt;TEntity&gt;"/> class.
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        public SqlDal(string connectionString)
        {
            ConnectionString = connectionString;
            IsDefaultDataUsedDuringCreate = true;
        }


        /// <summary>
        /// Creates the table.   Both CreateTable and CreateRecord
        /// both use the same process - the DalEventArgs.Type flag 
        /// determines which process to execute.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        /// <returns></returns>
        public virtual int CreateTable(object sender, EventArgs e)
        {
            var args = e as DalEventArgs ?? new DalEventArgs();
            args.Type = ProcessType.CreateTable;
            return Create(sender, args);
        }

        /// <summary>
        /// Deletes the table.   Both DeleteTable and DeleteRecord
        /// both use the same process - the DalEventArgs.Type flag 
        /// determines which process to execute.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        /// <returns></returns>
        public virtual int DropTable(object sender, EventArgs e)
        {
            var args = e as DalEventArgs ?? new DalEventArgs();
            args.Type = ProcessType.DropTable;
            return Delete(sender, args);
        }

        /// <summary>
        /// Creates the specified sender.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        /// <returns></returns>
        public virtual int Create(object sender, EventArgs e)
        {
            var args = e as DalEventArgs;
            if (args == null)
                throw new NotSupportedException(string.Format(
                    Messages.P0IsNotSupportedInP1,
                    e.GetType().Name,
                    GetType().FullName));

            var status = -1;
            using (var conn = new SqliteConnection(ConnectionString))
            {
                try
                {
                    conn.Open();

                    // INSERT RECORD
                    if ((ProcessType)args.Type == ProcessType.DataCreate)
                        return CreateRecord(conn, e);

                    // CREATE TABLE
                    if ((ProcessType)args.Type == ProcessType.CreateTable)
                    {
                        var sqlCreateCommand = SqlUtility.MakeSqlCreateTableString();

                        Logger.Log("Create(): " + sqlCreateCommand, Category.Debug, Priority.None);

                        return CreateTable(conn, sqlCreateCommand);
                    }

                }
                catch (Exception ex)
                {
                    throw new SqlException(ex.Message, ex);
                }

                status = 0;
            }
            return status;

        }
        


        // CRUD Methods


        /// <summary>
        /// Creates the record.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        /// <returns></returns>
        private int CreateRecord(SqliteConnection conn, EventArgs e)
        {
            var args = e as DalEventArgs;
            if (args == null)
                throw new NotSupportedException(string.Format(
                    Messages.P0IsNotSupported,
                    e.GetType().FullName));

            // If the primary key is -1 then it has not been initialized
            // we'll query the table for the highest primary key so that
            // we can auto-increment it
            if (args.PrimaryKey == -1)
            {
                // TODO: see if there is a "TOP 1" syntax available
                var getNextKeySql = string.Format(
                    "select {0} from {1} order by {0} DESC ",
                    args.PrimaryKeyField,
                    GetClassName());

                Logger.Log("{0}", getNextKeySql);

                // Execute the query to get the highest primary key
                var nextKeyCmd = new SqliteCommand(getNextKeySql, conn);
                var keyValue = nextKeyCmd.ExecuteScalar();

                // If it came back null (saved as null) then we'll use 0
                if (keyValue is string && keyValue.ToString() == "<null>")
                    keyValue = 0;

                // Set the last key
                args.PrimaryKey = Int32.Parse(keyValue.ToString());

                // Increment it it so that we're now the highest key
                args.PrimaryKey++;
            }

            // Create the insert statement
            PopulateEntityOnCreateRecord(args);
            var sqlCreateCommand = SqlUtility.MakeSqlInsertTableString(e);

            Logger.Log("CreateRecord(): " + sqlCreateCommand, Category.Debug, Priority.None);

            try
            {
                // Execute the insert command
                var cmd = new SqliteCommand(sqlCreateCommand, conn);
                cmd.ExecuteScalar();
            }
            catch (Exception ex)
            {
                args.Exception = ex;
                Logger.Log("{0}", ex.Message);
            }

            // Return the primary key value used
            return args.PrimaryKey;
        }

        /// <summary>
        /// Populates the entity on create record.   The contents of
        /// args.Data will be used to dynamically generate a Sql insert
        /// string which will be executed for the table
        /// </summary>
        /// <param name="args">The <see cref="MsdnDemo.Infrastructure.Portable.Events.DalEventArgs"/> instance containing the event data.</param>
        protected virtual void PopulateEntityOnCreateRecord(DalEventArgs args)
        {
        }

        private static object GetKey(DalEventArgs args)
        {
            if (args.PrimaryKey == -999)
                return string.Format("'{0}'",args.UniqueKey);
            return args.PrimaryKey;
        }

        /// <summary>
        /// Reads the specified record.  e.Data.PrimaryKey value
        /// is expected to hold the primary key of the record
        /// that you want to read.  If e.Data implements 
        /// ISupportInitialize it will execute its methods
        /// prior to, and after, populating the data instance.
        /// Executes PopulateEntityOnRead() on applicable DAL
        /// which is needs to be overridden to populate the
        /// instance as applicable.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        /// <returns></returns>
        public virtual TEntity Read(object sender, EventArgs e)
        {
            var args = e as DalEventArgs;
            if (args == null)
                throw new NotSupportedException(string.Format(
                    Messages.P0IsNotSupportedInP1,
                    e.GetType().Name,
                    GetType().FullName));

            using (var conn = new SqliteConnection(ConnectionString))
            {
                try
                {
                    conn.Open();

                    var readSql = string.Format(
                        "select * from {0} where {1}={2}",
                        GetClassName(),
                        args.PrimaryKeyField, GetKey(args));

                    Logger.Log("Read(): "+readSql, Category.Debug, Priority.None);

                    // Initialize the command
                    var cmd = new SqliteCommand(readSql, conn);

                    // Execute the reader and read the first (only) record
                    var rdr = cmd.ExecuteReader();
                    if (rdr.Read())
                    {
                        // Instantiate a new instance of entity
                        var data = Activator.CreateInstance<TEntity>();

                        if (data is ISupportInitialize)
                            ((ISupportInitialize)data).BeginInit();

                        // Let the parent populate it

                        Logger.Log("Read(): {0} populating record", GetType().Name);

                        PopulateEntityOnRead(rdr, data);

                        if (data is ISupportInitialize)
                            ((ISupportInitialize)data).EndInit();

                        // Raise conventional event
                        if (args.IsEventRaised && DataArrivedReadEvent != null)
                            DataArrivedReadEvent(this, new DataEventArgs
                            {
                                EventArgs = args,
                                Data = data,
                                Message = readSql,
                                Sender = this,
                                ProcessType = ProcessType.DataRead
                            });


                        return data;
                    }
                    return default(TEntity);
                }
                catch (Exception ex)
                {
                    throw new SqlException(ex.Message);
                }

            }
        }

 
        /// <summary>
        /// Reads the list.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        /// <returns></returns>
        public virtual IEnumerable<TEntity> ReadList(object sender, EventArgs e)
        {
            var args = e as DalEventArgs;
            if (args == null)
                throw new NotSupportedException(string.Format(
                    Messages.P0IsNotSupportedInP1,
                    e.GetType().Name,
                    GetType().FullName));

            using (var conn = new SqliteConnection(ConnectionString))
            {
                var returnList = new List<TEntity>();

                try
                {
                    conn.Open();
                    
                    var isRetry = false;
                    var lastRetryMessage = "";

                    RetryCommand:
                    
                    var readSql = string.Format(
                        "select * from {0}",
                        GetClassName());

                    ReadListWhereClauseSetter(args);
                    if (args.WhereClause != null)
                        readSql = string.Format("{0} where {1}", readSql, args.WhereClause);

                    Logger.Log(readSql, Category.Debug, Priority.None);


                    // Initialize the command
                    var cmd = new SqliteCommand(readSql, conn);

                    // Execute the reader and read the first (only) record
                    var rdr = cmd.ExecuteReader();

                    Logger.Log("{0} populating record", GetType().Name);
                    while (rdr.Read())
                    {
                        // Instantiate a new instance of entity
                        var data = Activator.CreateInstance<TEntity>();

                        if (data is ISupportInitialize)
                            ((ISupportInitialize)data).BeginInit();

                        try
                        {
                            // Let the parent populate it
                            PopulateEntityOnRead(rdr, data);
                        }
                        catch (IndexOutOfRangeException ex)
                        {
                            // If we haven't already retried for this error
                            // and the error is handled by parent class 
                            // then retry this command
                            if (!isRetry
                                && lastRetryMessage != ex.Message
                                && HandleIndexOutOfRangeException(ex))
                            {
                                lastRetryMessage = ex.Message;
                                isRetry = true;
                                goto RetryCommand;
                            }
                            throw; // If we get here then rethrow it
                        }

                        if (data is ISupportInitialize)
                            ((ISupportInitialize)data).EndInit();

                        // Add record to list
                        returnList.Add(data);
                    }

                    // Raise conventional event
                    if (DataArrivedReadEvent != null)
                        DataArrivedReadEvent(this, new DataEventArgs
                        {
                            EventArgs = args,
                            Data = returnList,
                            Message = readSql,
                            Sender = this,
                            ProcessType = ProcessType.DataRead
                        });

                    return returnList;
                }
                catch (Exception ex)
                {
                    throw new SqlException(ex.Message);
                }
            }
        }

        /// <summary>
        /// Handles the index out of range exception.
        /// </summary>
        /// <param name="ex">The ex.</param>
        /// <returns></returns>
        protected virtual bool HandleIndexOutOfRangeException(IndexOutOfRangeException ex)
        {
            return false;
        }

        /// <summary>
        /// Hook into ReadList to obtain WhereClause
        /// </summary>
        /// <param name="args">The <see cref="DalEventArgs"/> instance containing the event data.</param>
        protected virtual void ReadListWhereClauseSetter(DalEventArgs args)
        {
        }

        /// <summary>
        /// If the ReadList fails the parent has an opportunity
        /// to create a new one.  If parent creates table it must
        /// set DalEventArgs.IsTableCreated to true so that the
        /// business logic layer will know not to attempt to create
        /// it also
        /// </summary>
        /// <param name="e">The <see cref="DalEventArgs"/> instance containing the event data.</param>
        /// <returns></returns>
        public virtual void TableDoesNotExistHandler(DalEventArgs e)
        {
            
        }
        

        /// <summary>
        /// Updates the specified sender.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        /// <returns></returns>
        public virtual int Update(object sender, EventArgs e)
        {
            var args = e as DalEventArgs;
            if (args == null)
                throw new NotSupportedException(string.Format(
                    Messages.P0IsNotSupportedInP1,
                    e.GetType().Name,
                    GetType().FullName));

            using (var conn = new SqliteConnection(ConnectionString))
            {
                try
                {
                    conn.Open();

                    // Create the insert statement
                    var sqlUpdateCommand = SqlUtility.MakeSqlUpdateTableString(e);

                    Logger.Log(sqlUpdateCommand, Category.Debug, Priority.None);

                    // Execute the insert command
                    var cmd = new SqliteCommand(sqlUpdateCommand, conn);
                    cmd.ExecuteNonQuery();


                }
                catch (Exception ex)
                {
                    throw new SqlException(ex.Message);
                }
            }
            return StatusCode.Success;
        }
        

        /// <summary>
        /// Deletes the specified sender.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        /// <returns></returns>
        public virtual int Delete(object sender, EventArgs e)
        {
            var args = e as DalEventArgs;
            if (args == null)
                throw new NotSupportedException(string.Format(
                    Messages.P0IsNotSupportedInP1,
                    e.GetType().Name,
                    GetType().FullName));

            using (var conn = new SqliteConnection(ConnectionString))
            {
                try
                {
                    conn.Open();

                    // If we're dropping the table - drop it and return
                    if ((ProcessType)args.Type == ProcessType.DropTable)
                        return DeleteTable(conn);


                    // Default behavior is to delete the record
                    var readSql = string.Format(
                        "Delete from {0} where {1}={2}",
                        GetClassName(),
                        args.PrimaryKeyField,
                        args.PrimaryKey);

                    Logger.Log(readSql, Category.Debug, Priority.None);

                    // Initialize the command
                    var cmd = new SqliteCommand(readSql, conn);
                    var status = cmd.ExecuteNonQuery();

                }
                catch (Exception ex)
                {
                    throw new SqlException(ex.Message);
                }
            }
            return StatusCode.Success;
        }
        

        // Support methods

        private int CreateTable(SqliteConnection conn, string sqlCreateCommand)
        {
            var cmd = new SqliteCommand(sqlCreateCommand, conn);
            return cmd.ExecuteNonQuery();
        } 

        private int DeleteTable(SqliteConnection conn)
        {
            // create the sql statement to drop the table
            var deleteCommandSql = string.Format(
                "drop table {0}",
                GetClassName());

            Logger.Log(deleteCommandSql, Category.Debug, Priority.None);

            try
            {
                // Execute the sql command
                var cmd = new SqliteCommand(deleteCommandSql, conn);
                cmd.ExecuteNonQuery();
            }
            catch (SqliteConnectionException ex)
            {
                // If we are trying to drop a table and it doesn't
                // exist, it is not a real error
                if (ex.Message.Contains("no such table"))
                    return StatusCode.DoesNotExist;
                
                // Otherewise throw the error
                throw;
            }
            return StatusCode.Success;
        }
        

        /// <summary>
        /// Gets the name of the class.
        /// </summary>
        /// <returns></returns>
        private string GetClassName()
        {
            // We don't want ViewModel in our table names
            return typeof(TEntity).Name.Replace("ViewModel","");
        }
        

        /// <summary>
        /// Sets the connection string.
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        public virtual void SetConnectionString(string connectionString)
        {
            ConnectionString = connectionString;
        }


        /// <summary>
        /// Populates the entity when data arrives.
        /// </summary>
        /// <param name="rdr">The RDR.</param>
        /// <param name="data">The data.</param>
        protected virtual void PopulateEntityOnRead(IDataReader rdr, TEntity data)
        {
            throw new NotImplementedException(
                "You must override PopulateEntityWhenDataArrived in " + GetType().Name);
        }


        /// <summary>
        /// Updates the specified sender.
        /// </summary>
        /// <param name="sqlCommand">The SQL command.</param>
        /// <returns></returns>
        public virtual bool SqlExec(string sqlCommand)
        {
            using (var conn = new SqliteConnection(ConnectionString))
            {
                try
                {
                    conn.Open();

                    Logger.Log(sqlCommand, Category.Debug, Priority.None);

                    // Execute the insert command
                    var cmd = new SqliteCommand(sqlCommand, conn);
                    cmd.ExecuteNonQuery();
                }
                catch (Exception ex)
                {
                    throw new SqlException(ex.Message);
                }
            }
            return true;
        }


    }
}
