﻿using System.Linq;
using System.Text;
using System;
using MsdnDemo.Infrastructure.Portable.Attributes;
using MsdnDemo.Infrastructure.Portable.Events;

namespace MsdnDemo.LayerDataAccess.Sqlite
{
    /// <summary>
    /// Sql utility extensions
    /// </summary>
    public static class SqlUtilityExtension
    {
        /// <summary>
        /// Makes the SQL insert table string.
        /// </summary>
        /// <param name="helper">The helper.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        /// <returns></returns>
        public static string MakeSqlInsertTableString(this SqlUtility helper, EventArgs e)
        {
            var args = e as DalEventArgs;
            if (args == null)
                throw new NotSupportedException(string.Format(
                    "{0} is not supported",
                    e.GetType().FullName));

            var delimiter = "";

            // Get table name - we don't want ViewModel to be in the name
            var tablename = helper.Type.Name.Replace("ViewModel", "");

            var sb = new StringBuilder("insert into ");
            sb.AppendFormat("{0} ", tablename);

            var sbFieldList = new StringBuilder();
            var sbValueList = new StringBuilder();

            foreach (var prop in from prop in helper.Type.GetProperties()
                                 let attributes = prop.GetCustomAttributes(typeof(DataFieldExcludeAttribute), true)
                                 where attributes.Length <= 0
                                 select prop)
            {
                var name = prop.Name;
                var value = GetStringFromValue(prop.GetValue(args.Data, null));

                // If the current field name matches the primary key 
                if (name.ToLower() == args.PrimaryKeyField.ToLower())
                {
                    // Only use if primary key was initialized in args
                    if (args.PrimaryKey != -1)
                        value = args.PrimaryKey;

                    if (value is Int32 && (int)value == 0)
                        throw new NotSupportedException(
                            "DEVELOPER: You cannot have a primary key value of 0\r\n"
                            +"Something didn't get initialized");
                }

                sbFieldList.AppendFormat("{0} {1}", delimiter, name);
                sbValueList.AppendFormat("{0} {1}", delimiter, value);
                delimiter = ",";
            }

            sb.AppendFormat("({0}) values ({1})", sbFieldList, sbValueList);

            return sb.ToString();
        }


        /// <summary>
        /// Makes the SQL update table string.
        /// </summary>
        /// <param name="helper">The helper.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        /// <returns></returns>
        public static string MakeSqlUpdateTableString(this SqlUtility helper, EventArgs e)
        {
            var args = e as DalEventArgs;
            if (args == null)
                throw new NotSupportedException(string.Format(
                    "{0} is not supported",
                    e.GetType().FullName));

            var delimiter = "";

            // Get table name - we don't want ViewModel to be in the name
            var tablename = helper.Type.Name.Replace("ViewModel","");

            var sb = new StringBuilder("update ");
            sb.AppendFormat("{0} set ", tablename);

            foreach (var prop in (from prop in helper.Type.GetProperties()
                                  let attributes = prop.GetCustomAttributes(typeof (DataFieldExcludeAttribute), true)
                                  where attributes.Length <= 0
                                  select prop).Where(prop => prop.Name.ToLower() != "id"))
            {
                sb.AppendFormat("{0} {1}={2}",
                                delimiter,
                                prop.Name,
                                GetStringFromValue(prop.GetValue(args.Data, null)));

                delimiter = ",";
            }

            sb.AppendFormat(" where {0}={1}", args.PrimaryKeyField, args.PrimaryKey);
            return sb.ToString();
        }

        private static object GetStringFromValue(object data)
        {
            if (data is Int32)
                return data;
            return string.Format("'{0}'", data);
        }

        /// <summary>
        /// Makes the SQL create table string.
        /// </summary>
        /// <param name="helper">The helper.</param>
        /// <returns></returns>
        public static string MakeSqlCreateTableString(this SqlUtility helper)
        {
            var delimiter = "";

            var sb = new StringBuilder("create table ");

            // We don't want ViewModel to be in the name
            sb.AppendFormat("{0} (", helper.Type.Name.Replace("ViewModel", ""));

            foreach (var prop in from prop in helper.Type.GetProperties()
                                 let attributes = prop.GetCustomAttributes(typeof (DataFieldExcludeAttribute), true)
                                 where attributes.Length <= 0
                                 select prop)
            {
                sb.AppendFormat("{0} {1}",
                                delimiter,
                                Field(prop.Name, prop.PropertyType.Name));

                delimiter = ",";
            }

            sb.AppendFormat(" )");

            return sb.ToString();
        }

        private static string Field(string fieldName, string typeName)
        {
            if (fieldName.ToLower().Trim().Equals("id"))
                return "id int primary key";

            return string.Format("{0} {1}", fieldName, Parse(typeName));
        }

        private static string Parse(string typeName)
        {
            if (typeName.ToLower().Contains("int"))
                return "int";

            return "text";
        }

    }
}
