﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using Microsoft.Practices.Prism.Events;
using Microsoft.Practices.Prism.Modularity;
using Microsoft.Practices.Unity;
using MsdnDemo.Events;
using MsdnDemo.Infrastructure.Portable.Base;
using MsdnDemo.Infrastructure.Portable.Constants;
using MsdnDemo.Infrastructure.Portable.Interfaces;
using MsdnDemo.Infrastructure.Portable.Logger;
using MsdnDemo.Infrastructure.Portable.Events;
using MsdnDemo.Infrastructure.Portable.MvpVmBase;
using MsdnDemo.Infrastructure.Portable.Types;
using MsdnDemo.Phone.Events;

namespace MsdnDemo.Modules
{
    public class ApplicationControllerModule : ModuleBase, IModule
    {
        #region Fields
        private readonly List<ViewEventArgs> _viewList = new List<ViewEventArgs>();

        private readonly Dictionary<string, string[]> _moduleRoles = new Dictionary<string, string[]>();

        private object DefaultView { get; set; }
        #endregion 

        #region Properties 
        [Dependency]
        public new ILogger Logger
        {
            get { return base.Logger; }
            set { base.Logger = value; }
        }

        [Dependency]
        public IEventAggregator EventAggregator { get; set; }

        [Dependency]
        public IUnityContainer Container { get; set; }
        #endregion 

        /// <summary>
        /// Registers the types.
        /// </summary>
        protected override void RegisterTypes()
        {
            base.RegisterTypes();

            EventAggregator.GetEvent<ModuleEvent>()
                .Subscribe(HandleModuleEvents, true);

            EventAggregator.GetEvent<ViewEvent>()
                .Subscribe(HandleViewEvents, true);
        }

        /// <summary>
        /// Gets the type from the supplied name
        /// </summary>
        /// <param name="viewName">Name of the view.</param>
        /// <returns></returns>
        public Type GetTypeFromName(string viewName)
        {
            return _viewList
                .Where(item => item.View.GetType().Name == viewName)
                .Select(item => item.View.GetType())
                .FirstOrDefault();
        }

        /// <summary>
        /// Activates the view.
        /// </summary>
        /// <param name="viewName">Name of the view.</param>
        public void ActivateView(string viewName)
        {
            Logger.Log("ViewName = [{0}]", viewName);


            // Activate this view (deactivates others to include Login)
            EventAggregator.GetEvent<ViewEvent>()
                .Publish(new ViewEventArgs
                {
                    ProcessType = ProcessType.ActivateView,
                    ViewTypeToActivate = GetTypeFromName(viewName)
                });
        }

        /// <summary>
        /// Handles the view events.
        /// </summary>
        /// <param name="e">The <see cref="ViewEventArgs"/> 
        /// instance containing the event data.</param>
        public void HandleViewEvents(ViewEventArgs e)
        {
            Logger.Log("ProcessType:[{0}] View:[{1}] ViewTypeToActivate:{2}  ViewNameToActivate:{3}",
                e.ProcessType, e.View, e.ViewTypeToActivate, e.ViewNameToActivate);

            switch (e.ProcessType)
            {
                // REGISTER A VIEW 
                case ProcessType.NewView:
                    e.ViewTypeToActivate = e.View.GetType();
                    _viewList.Add(e);
                    break;

                case ProcessType.ActivateView:
                    // Activate by name (recursively)
                    if (e.ViewTypeToActivate == null && e.ViewNameToActivate != null)
                    {
                        Logger.Log("ViewNameToActivate = [{0}]", e.ViewNameToActivate);
                        ActivateView(e.ViewNameToActivate);
                        return;
                    }

                    if (e.ViewTypeToActivate == null)
                    {
                        Logger.Log("ViewTypeToActivate was null - using default view");
                        e.ViewTypeToActivate = GetTypeFromName("SecurityView");
                    }

                    Logger.Log("ViewTypeToActivate = [{0}]", e.ViewTypeToActivate);

                    // Activate by type - ensure only one view is visible at any given time
                    foreach (var item in _viewList)
                    {
                        var view = item.View as UserControl;
                        view.Visibility = item.View.GetType().Name == e.ViewTypeToActivate.Name
                            ? Visibility.Visible
                            : Visibility.Collapsed;

                        if(view.Visibility==Visibility.Visible)
                        {
                            var viewModel = view.DataContext as IMvpVmViewModel;
                            if (viewModel != null)
                                viewModel.Presenter.ViewActivated(e);

                        }
                    }
                    break;
            }
        }

        /// <summary>
        /// Handles the module events. Loads modules on demand - as applicable
        /// </summary>
        /// <param name="e">The <see cref="MsdnDemo.Infrastructure.Portable.Events.ModuleEventArgs"/> 
        /// instance containing the event data.</param>
        public void HandleModuleEvents(ModuleEventArgs e)
        {
            // We're only interest in the login event
            if (e.EventType != ModuleEventType.Login)
                return;

            // Let's get the security view model so that we can
            // examine the login states
            var securityViewModel = Container.Resolve<ISecurityViewModel>();

            // Don't load any modules if the user did not authenticate
            // Later, we can load a guest user screen if required
            if (!securityViewModel.IsAuthenticated)
                return;

            // Get module catalog so that we can get modules to load on-demand
            var moduleCatalog = Container.Resolve<IModuleCatalog>();

            // We'll need the module manager to dynamically load the modules
            var moduleManager = Container.Resolve<IModuleManager>();

            // If after processing this is still false then the user
            // is not authorized to access module.  A message event
            // will be raised below so that user can be notified
            var isAuthorized = false;

            // If not logged in then we won't have views to activate
            // because they have not yet been loaded (on demand)
            if (_moduleRoles.ContainsKey(e.ModuleName))
            {   
                // Get the module roles
                var roles = _moduleRoles[e.ModuleName];
                if (securityViewModel.IsInRole(roles))
                {
                    // If the user is authorized access to module activate it
                    ActivateView(e.PayLoad.ToString());
                    isAuthorized = true;
                }
            }


            // Iterate through each module and load all of the modules
            // configured to be loaded on demand as it is requested
            foreach (var catalog in moduleCatalog.Modules)
                if (catalog.InitializationMode == InitializationMode.OnDemand)
                {
                    string[] roles = {};
                    var moduleName = catalog.ModuleName;
                    var roleOffset = catalog.ModuleName.IndexOf(':');
                    var isModuleAuthorized = false;

                    // If there are roles assigned to the module in the
                    // GwnBootstrapper
                    if(roleOffset>0)
                    {
                        // If roles are assigned to module name they will
                        // be after a colon, e.g., MainModule:User,Admin
                        moduleName = catalog.ModuleName.Substring(0, roleOffset);
                        
                        // this isn't the module we are requesting so we won't
                        // load it - only process requested module
                        if (moduleName != e.ModuleName)
                            continue;

                        // Split the module roles into the format expected by
                        // the IsInRole method
                        roles = catalog.ModuleName.Substring(roleOffset + 1).Split(',');

                        // Is the user authorized to this module?
                        isModuleAuthorized = securityViewModel.IsInRole(roles);
                    }

                    if (isModuleAuthorized) // Only load module if authorized
                    {
                        // Add it to our module roles collection so that we
                        // can verify that future access to the module is 
                        // authorized (user could login as someone else)
                        _moduleRoles.Add(moduleName, roles);

                        // This will only load/launch one time - once loaded
                        // the ActivateView() method above will be responsible
                        // for activating the view.
                        catalog.ModuleName = moduleName;
                        moduleManager.LoadModule(moduleName);

                        isAuthorized = true;
                    }
                }

            // If not authorized then nothing will have happened - publish 
            // message so that subscribers can notify user
            if (!isAuthorized)
                EventAggregator.GetEvent<MessageEvent>()
                    .Publish(new MessageEventArgs
                                 {
                                     Message = "User not authorized access\r\nto this module!",
                                     Sender = this,
                                     StatusCode = StatusCode.NotAuthorized
                                 });

        }
    }
}
