﻿using System;
using System.Windows;
using Microsoft.Practices.Prism.Modularity;
using Microsoft.Practices.Prism.Regions;
using Microsoft.Practices.Prism.UnityExtensions;
using Microsoft.Practices.Unity;
using MsdnDemo.Infrastructure.Portable.Interfaces;
using MsdnDemo.Infrastructure.Portable.Configuration;
using MsdnDemo.Infrastructure.Portable.Logger;
using MsdnDemo.Modules;

namespace MsdnDemo.Prism
{
    /// <summary>
    /// Bootstrapper
    /// </summary>
    public class GwnBootstrapper : UnityBootstrapper, IBootstrapper
    {
        private IShell _shell;
        private ILogger _logger;

        public Action<IShell> InitializeShellCallBack = shell => { };

        /// <summary>
        /// Initializes a new instance of the <see cref="GwnBootstrapper"/> class.
        /// </summary>
        public GwnBootstrapper(){}

        /// <summary>
        /// Initializes a new instance of the <see cref="GwnBootstrapper"/> class.
        /// </summary>
        /// <param name="container">The container.</param>
        public GwnBootstrapper(IUnityContainer container)
        {
            Container = container;
        }

        /// <summary>
        /// Configures the <see cref="IModuleCatalog"/> used by Prism.
        /// </summary>
        protected override void ConfigureModuleCatalog()
        {
            base.ConfigureModuleCatalog();

            var moduleCatalog = ModuleCatalog as ModuleCatalog;
            if (moduleCatalog == null)
                throw new InvalidOperationException("Module Catalog is not available in GwnBootstrapper");

            // The application controller will respond to the Login event (ModuleEvent) and
            // load all of the modules configured to load on-demand
            moduleCatalog.AddModule(typeof(ApplicationControllerModule));
           
            // We're going to use the default security module (in the Gwn.Infrastructure project)
            moduleCatalog.AddModule(typeof(SecurityModule));

            // Will only be loaded on successful login
            LoadModuleOnDemand(typeof (MainModule), "User,Admin");
            LoadModuleOnDemand(typeof (GuestModule), "User,Guest");
        }

        private void LoadModuleOnDemand(Type moduleType, string roles)
        {
            ((ModuleCatalog) ModuleCatalog).AddModule(
                new ModuleInfo
                    {
                        ModuleName = moduleType.Name+":"+roles,
                        ModuleType = moduleType.AssemblyQualifiedName,
                        InitializationMode = InitializationMode.OnDemand
                    });
        }


        /// <summary>
        /// Used by the Phone so that we can get a reference to the 
        /// Shell since the MainPage.xaml.cs is the starting point
        /// </summary>
        /// <param name="shell">The shell.</param>
        public void Run(IShell shell)
        {
            _shell = shell;
            Run();
        }

        /// <summary>
        /// Configures the <see cref="IUnityContainer"/>. 
        /// May be overwritten in a derived class to add specific
        /// type mappings required by the application.
        /// </summary>
        protected override void ConfigureContainer()
        {
            base.ConfigureContainer();

            // Global registrations
            Container
                .RegisterType<IAppConfig,AppConfig>(new ContainerControlledLifetimeManager())
                .RegisterType<ILogger,DebugLogger>()
                ;
            
            _logger = Container.Resolve<ILogger>();
            _logger.Log("{0}.ConfigureContainer()",GetType());
            
        }

        /// <summary>
        /// Creates the shell or main window of the application.
        /// </summary>
        /// <returns>The shell of the application.</returns>
        /// <remarks>
        /// If the returned instance is a <see cref="DependencyObject"/>, the
        /// <see cref="GwnBootstrapper"/> will attach the default <seealso cref="IRegionManager"/> of
        /// the application in its <see cref="RegionManager.RegionManagerProperty"/> attached property
        /// in order to be able to add regions by using the <seealso cref="RegionManager.RegionNameProperty"/>
        /// attached property from XAML.
        /// </remarks>
        protected override DependencyObject CreateShell()
        {
           _logger.Log("{0}.CreateShell()", GetType());

            // _shell will not be null for Windows Phone
            if (_shell == null)
                _shell = Container.Resolve<MainPage>();

            // Register an instance of the shell with container
            Container.RegisterInstance<IShell>(_shell);

            return (DependencyObject) _shell;
        }

        /// <summary>
        /// Initializes the shell.
        /// </summary>
        protected override void InitializeShell()
        {
           // Logger.LogDebug("CreateShell", GetType());
            base.InitializeShell();

            // Provide application a hook into Initialize
            // shell - each platform has different responsibilities
            InitializeShellCallBack((IShell)Shell);

        }
    }
}
