﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Input;
using System.Windows.Interactivity;
using Microsoft.Practices.Prism.Interactivity;
using MsdnDemo.Infrastructure.Portable.Events;

namespace MsdnDemo.TriggerActions
{
    /// <summary>
    /// Modified the following to emulate Desktop/Silverlight environments
    /// http://compiledexperience.com/blog/posts/blendable-mvvm-commands-and-behaviors
    /// Uses Prism.Interactivity.Phone 
    /// </summary>
    public class ExecuteCommandAction : TriggerAction<FrameworkElement>
    {
        private readonly BindingListener _commandListener;
        private readonly BindingListener _commandParameterListener;


        /// <summary>
        /// Gets or sets the command.
        /// </summary>
        /// <value>The command.</value>
        public Binding Command
        {
            get { return (Binding)GetValue(CommandProperty); }
            set { SetValue(CommandProperty, value); }
        }

        /// <summary>
        /// Command property
        /// </summary>
        public static readonly DependencyProperty CommandProperty =
            DependencyProperty.Register("Command",
                typeof(Binding),
                typeof(ExecuteCommandAction),
                new PropertyMetadata(null,
                    (d, e) => ((ExecuteCommandAction)d).OnCommandBindingChanged(e))); 

        /// <summary>
        /// Gets or sets the command parameter.
        /// </summary>
        /// <value>The command parameter.</value>
        public Binding CommandParameter
        {
            get { return (Binding)GetValue(CommandParameterProperty); }
            set { SetValue(CommandParameterProperty, value); }
        }

        /// <summary>
        /// CommandParameter property
        /// </summary>
        public static readonly DependencyProperty CommandParameterProperty =
            DependencyProperty.Register("CommandParameter",
                typeof(Binding),
                typeof(ExecuteCommandAction),
                new PropertyMetadata(null,
                    (d, e) => ((ExecuteCommandAction)d).OnCommandParameterBindingChanged(e))); 


        /// <summary>
        /// Initializes a new instance of the <see cref="ExecuteCommandAction"/> class.
        /// </summary>
        public ExecuteCommandAction()
        {
            _commandListener = new BindingListener();
            _commandParameterListener = new BindingListener();
        } 

        /// <summary>
        /// Called after the action is attached to an AssociatedObject.
        /// </summary>
        protected override void OnAttached()
        {
            base.OnAttached();

            _commandListener.Element = AssociatedObject;
            _commandParameterListener.Element = AssociatedObject;
        } 

        /// <summary>
        /// Called when the action is being detached from its 
        /// AssociatedObject, but before it has actually occurred.
        /// </summary>
        protected override void OnDetaching()
        {
            base.OnDetaching();

            _commandListener.Element = null;
            _commandParameterListener.Element = null;
        } 


        /// <summary>
        /// Invokes the action.
        /// </summary>
        /// <param name="parameter">The parameter to the action. 
        /// If the Action does not require a parameter, the parameter 
        /// may be set to a null reference.</param>
        protected override void Invoke(object parameter)
        {
            var command = _commandListener.Value as ICommand;

            if (command == null)
                return;

            var args = new DelegateCommandEventArgs();

            #region <Style TargetType="Button" x:Key="ButtonStyle"
            /*<Setter Property="Template">
            <Setter.Value>
                <ControlTemplate TargetType="Button">
                    <StackPanel Orientation="Horizontal">
                        <Border CornerRadius="20" 
                                    BorderBrush="{TemplateBinding BorderBrush}"
                                    BorderThickness="{TemplateBinding BorderThickness}"
                                    Padding="0" Margin="20">
                            <Button
                                    BorderThickness="0"
                                    Margin="{TemplateBinding Margin}"
                                    Height="{TemplateBinding Height}"
                                    Width="{TemplateBinding Width}"
                                    Background="{TemplateBinding Background}"
                                    Foreground="{TemplateBinding Foreground}"
                                    Tag="{Binding RelativeSource={RelativeSource TemplatedParent }}"
                                    >
                                <Custom:Interaction.Triggers>
                                    <Custom:EventTrigger EventName="Click" >
                                        <TriggerActions:ExecuteCommandAction 
                                                Command="{Binding ButtonCommand}" 
                                                CommandParameter="{Binding RelativeSource={RelativeSource TemplatedParent }}"/>
                                    </Custom:EventTrigger>
                                </Custom:Interaction.Triggers>
                                <Button.Content>
                                    <ContentPresenter/>
                                </Button.Content>
                            </Button>
                        </Border>
                    </StackPanel>
                </ControlTemplate>
            </Setter.Value>
            </Setter>
            </Style>*/
            #endregion

            // Unlike Desktop/Silverlight we can't get the TemplatedParent button
            // reference in our ButtonStyle, we can only get the "template" button.
            // However the template button Tag can bind to the actual button so 
            // we'll pull it out of the tag and update sender with it so it will 
            // be consistent with Desktop and Silverlight

            var btn = _commandListener.Element as Button;
            if (btn != null)
            {
                var button = btn.Tag as Button;
                if (button != null)     // For template
                    args.Sender = button;
                else
                    args.Sender = btn;  // for default use
            }
            else // otherwise pass in the element
                args.Sender = _commandListener.Element;

            if (command.CanExecute(args))
                command.Execute(args);
        } 

        private void OnCommandBindingChanged(DependencyPropertyChangedEventArgs e)
        {
            _commandListener.Binding = (Binding)e.NewValue;
        }

        private void OnCommandParameterBindingChanged(DependencyPropertyChangedEventArgs e)
        {
            _commandParameterListener.Binding = (Binding)e.NewValue;
        }
    }
}
