﻿#define SqliteTest 
using System.Collections.Generic;
using Microsoft.Practices.Unity;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using MsdnDemo.Events;
using MsdnDemo.Infrastructure.Portable.Constants;
using MsdnDemo.Infrastructure.Portable.Entities;
using MsdnDemo.Infrastructure.Portable.Events;
using MsdnDemo.Infrastructure.Portable.Interfaces;
using MsdnDemo.Infrastructure.Portable.Types;
using MsdnDemo.LayerBusinessLogic;
using MsdnDemo.LayerBusinessLogic.Interfaces;
using MsdnDemo.LayerDataAccess;
using MsdnDemo.Mvc3.Tests.Base;
using MsdnDemo.Mvc3.Tests.Mocks;

namespace MsdnDemo.Mvc3.Tests.Layers
{
    [TestClass]
    public class BusinessLayerFixture : FixtureBase
    {
        private IBll _testBll;
        private RoleEntity _roleEntity;

        [TestInitialize]
        public void InitializeTest()
        {
            // Registers default values (normally supplied by Prism)
            Initialize();

            Container 
#if SqliteTest
                // Roles
                .RegisterType<IRolesBll, RoleBll>()
                .RegisterType<IDal<RoleEntity>, RolesDal>()
                // User
                .RegisterType<IUserBll, UserBll>()
                .RegisterType<IDal<UserEntity>, UserDal>()
                // User Role
                .RegisterType<IUserRoleBll, UserRoleBll>()
                .RegisterType<IDal<UserRoleEntity>, UserRoleDal>()
                // Mock
                .RegisterType<IMockBll, MockBll>()
                .RegisterType<IDal<MockEntity>, MockDal>()
                ;
#else
            // TODO: Sql server registrations go here....
#endif

            _testBll = Container.Resolve<IRolesBll>();
            _testBll.Initialize();

            // Record to create
            _roleEntity = new RoleEntity
            {
                Role = "Admin",
                Description = "Administrator"
            };

        }


        [TestMethod, Owner("Layers")]
        public void CanCreateUpdateReadDeleteRecord()
        {
            // Add to args, args.PrimaryKey will be set to 
            // to the primary key for the new record
            var args = new DalEventArgs { Data = _roleEntity };

            // Create the record
            var status = _testBll.Create(args);

            // Assert success
            Assert.AreEqual(StatusCode.Success, status);
            Assert.AreEqual(1, args.PrimaryKey);

            // Read the record
            var readArgs = new DalEventArgs { PrimaryKey = 1 };
            status = _testBll.Read(readArgs);
            Assert.AreEqual(StatusCode.Success, status);
            Assert.AreEqual("Admin", readArgs.GetResults<RoleEntity>().Role);
            Assert.AreEqual("Administrator", readArgs.GetResults<RoleEntity>().Description);

            var data = readArgs.Results as RoleEntity;
            Assert.IsNotNull(data);
            
            var primaryKey = data.Id;
            data.Description = "New Description";
            status = _testBll.Update(new DalEventArgs
                                         {
                                             PrimaryKey = primaryKey,
                                             Data = data
                                         });
            Assert.AreEqual(StatusCode.Success, status);

            // Read the record - start with a clean event args
            // so we can test the Data object upon return
            readArgs = new DalEventArgs { PrimaryKey = primaryKey };
            status = _testBll.Read(readArgs);
            Assert.AreEqual(StatusCode.Success, status);
            Assert.AreEqual("Admin", readArgs.GetResults<RoleEntity>().Role);
            Assert.AreEqual("New Description", readArgs.GetResults<RoleEntity>().Description);

            // Create another record
            status = _testBll.Create(new DalEventArgs
                                         {
                                              Data = new RoleEntity{ Role="User", Description="User Role"}
                                         });
            Assert.AreEqual(StatusCode.Success, status);

            // Get list
            var listArgs = new DalEventArgs();
            status = _testBll.List(listArgs);
            Assert.AreEqual(StatusCode.Success, status);
            Assert.IsNotNull(listArgs.Results);

            var list = new List<RoleEntity>(listArgs.GetResults<IEnumerable<RoleEntity>>());
            Assert.AreEqual(2, list.Count);

            // Delete first record
            status = _testBll.Delete(new DalEventArgs {PrimaryKey = 1});
            Assert.AreEqual(StatusCode.Success, status);

            // Get list
            status = _testBll.List(listArgs);
            Assert.AreEqual(StatusCode.Success, status);
            list = new List<RoleEntity>(listArgs.GetResults<IEnumerable<RoleEntity>>());
            Assert.AreEqual(1, list.Count);
            Assert.AreEqual("User", list[0].Role);
            
            // Cleanup
            _testBll.DropTable();
        }

        [TestMethod, Owner("Layers")]
        public void CanCreateReadRecord()
        {
            _testBll.DropTable();

            // Add to args, args.PrimaryKey will be set to 
            // to the primary key for the new record
            var args = new DalEventArgs { Data = _roleEntity };

            // Create the record
            var status = _testBll.Create(args);

            // Assert success
            Assert.AreEqual(StatusCode.Success, status);
            Assert.AreEqual(1, args.PrimaryKey);

            // Read the record
            var readArgs = new DalEventArgs {PrimaryKey = 1};
            status = _testBll.Read(readArgs);
            Assert.AreEqual(StatusCode.Success, status);
            Assert.AreEqual("Admin", readArgs.GetResults<RoleEntity>().Role);
            Assert.AreEqual("Administrator", readArgs.GetResults<RoleEntity>().Description);

            // Read the record
            readArgs = new DalEventArgs { PrimaryKey = 2 };
            status = _testBll.Read(readArgs);
            Assert.AreEqual(StatusCode.NoData, status);

            // Cleanup
            _testBll.DropTable();
        }

        [TestMethod, Owner("Layers")]
        public void CanCreateRecord()
        {
            // Cleanup
            _testBll.DropTable();

            // Add to args, args.PrimaryKey will be set to 
            // to the primary key for the new record
            var args = new DalEventArgs {Data = _roleEntity};
            
            // Create the record
            var status = _testBll.Create(args);

            // Assert success
            Assert.AreEqual(StatusCode.Success, status);
            Assert.AreEqual(1, args.PrimaryKey);

            // Cleanup
            _testBll.DropTable();
        }

        [TestMethod, Owner("Layers")]
        public void CanDropTable()
        {
            var status = _testBll.CreateTable();

            Assert.IsTrue(status == StatusCode.Success || status == StatusCode.AlreadyExists);

            status = _testBll.DropTable();
            Assert.AreEqual(StatusCode.Success, status);

            status = _testBll.DropTable();
            Assert.AreEqual(StatusCode.DoesNotExist, status);

        }

        /// <summary>
        /// Errors from dal are handled by BLL.
        /// </summary>
        [TestMethod, Owner("Layers")]
        public void ErrorFromDalAreHandledByBll()
        {
            _testBll.DropTable();

            MessageEventArgs args = null;

            // Subscribe to message event and update
            // args with message event arg 
            EventAggregator.GetEvent<MessageEvent>()
                .Subscribe(e => args = e);

            // Force an error
            _testBll.IsFailRequiredForUnitTest = true; ;
            var status = _testBll.CreateTable();

            Assert.IsNotNull(args);
            Assert.IsTrue(args.IsError);
            Assert.AreEqual("Bogus error message", args.Exception.Message);
            Assert.AreEqual(StatusCode.TableCreateError, args.ErrorCode);
            Assert.AreEqual(ProcessType.CreateTable, (ProcessType)args.Type);

            status = _testBll.DropTable();
            Assert.AreEqual(StatusCode.TableCreateError, status);
        }

        /// <summary>
        /// Tests the BLL can create table.
        /// </summary>
        [TestMethod, Owner("Layers")]
        public void TestBllCanCreateTable()
        {
            MessageEventArgs args = null;

            // Subscribe to message event and update
            // args with message event arg 
            EventAggregator.GetEvent<MessageEvent>()
                .Subscribe(e => args = e);

            var status = _testBll.CreateTable();

            if (args != null) // Table may already exists (from prior test)
            {
                Assert.AreEqual(status, args.ErrorCode);
                Assert.AreEqual(StatusCode.TableCreateError, args.ErrorCode);
                Assert.AreEqual(StatusCode.AlreadyExists, args.StatusCode);
            }
            else
                Assert.AreEqual(StatusCode.Success, status);

            status = _testBll.DropTable();
            Assert.AreEqual(StatusCode.Success, status);
        }

        [TestMethod, Owner("Layers")]
        public void CanResolveBllAndSetSecurityInfo()
        {
            var securityVm = Container.Resolve<ISecurityViewModel>();
            securityVm.IsAuthenticated = true;
            securityVm.Login = "Bill";
            securityVm.Roles = new List<string> { "Admin", "User" };

            var bll = Container.Resolve<IUserBll>();
            Assert.IsNotNull(bll);

            bll.Initialize();

            Assert.IsTrue(securityVm.IsInRole("Admin"));
            Assert.IsTrue(securityVm.IsInRole("User"));
            Assert.IsFalse(securityVm.IsInRole("Guest"));


            bll.DropTable();
        }

        /// <summary>
        /// Determines whether this instance [can create read update delete list BLL].
        /// </summary>
        [TestMethod, Owner("Layers")]
        public void CanRegisterAndInitializeMockBll()
        {
            var securityVm = Container.Resolve<ISecurityViewModel>();
            securityVm.IsAuthenticated = true;

            var bll = Container.Resolve<IMockBll>();
            Assert.IsNotNull(bll);

            bll.Initialize();
        }
    }
}
