﻿#define SqliteTest
using System.Collections.Generic;
using Microsoft.Practices.Unity;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using MsdnDemo.Infrastructure.Portable.Constants;
using MsdnDemo.Infrastructure.Portable.Entities;
using MsdnDemo.Infrastructure.Portable.Events;
using MsdnDemo.Infrastructure.Portable.Interfaces;
using MsdnDemo.LayerBusinessLogic;
using MsdnDemo.LayerBusinessLogic.Interfaces;
using MsdnDemo.LayerDataAccess;
using MsdnDemo.Mvc3.Tests.Base;
using MsdnDemo.Mvc3.Tests.Mocks;

namespace MsdnDemo.Mvc3.Tests.Layers
{
    [TestClass]
    public class UserFixture : FixtureBase
    {
        private IUserBll _testBll;
        private IMockBll _mockBll;

        [TestInitialize]
        public void InitializeTest()
        {
            // Registers default values (normally supplied by Prism)
            Initialize();

            Container
#if SqliteTest
                // Roles
                .RegisterType<IRolesBll, RoleBll>()
                .RegisterType<IDal<RoleEntity>, RolesDal>()
                // User
                .RegisterType<IUserBll, UserBll>()
                .RegisterType<IDal<UserEntity>, UserDal>()
                // User Role
                .RegisterType<IUserRoleBll, UserRoleBll>()
                .RegisterType<IDal<UserRoleEntity>,UserRoleDal>()
                // Mock
                .RegisterType<IMockBll, MockBll>()
                .RegisterType<IDal<MockEntity>, MockDal>()
                ;
#else
                // TODO: Sql server registrations go here....
#endif

            _testBll = Container.Resolve<IUserBll>();
            _testBll.Initialize();

            _mockBll = Container.Resolve<IMockBll>();
            _mockBll.Initialize();

        }

        [TestMethod]
        public void CannotAddDuplicateUser()
        {
            const string login = "NewUser@MyDomain.com";
            const string userRoles = "User,Guest";


            var userToCreate = new UserEntity
                {
                    LastName = "User",
                    FirstName = "New",
                    Password = "MyPassword",
                    PrimaryEmail = login,
                };

            // Create the user with the role of User
            var status = _testBll
                .CreateUsersWithRoles(userToCreate, userRoles);
           
            Assert.AreEqual(StatusCode.Success, status);

            // Create a duplicate user
            var args = new DalEventArgs
                           {
                               PrimaryKey=userToCreate.Id,
                               Data = userToCreate
                           };
            status = _testBll.Create(args);

            Assert.AreEqual(StatusCode.UniquenessConstraint, status);
            Assert.AreEqual(args.PrimaryKey, userToCreate.Id);

            _testBll.DropTable();
        }

        [TestMethod]
        public void CanDeleteRoleFromUser()
        {
            const string login = "NewUser@MyDomain.com";
            const string userRoles = "User,Guest";

            var securityViewModel = Container.Resolve<ISecurityViewModel>();

            // Create the user with the role of User
            _testBll.CreateUsersWithRoles(
                new UserEntity {
                        LastName = "User",
                        FirstName = "New",
                        Password = "MyPassword",
                        PrimaryEmail = login,
                    }, userRoles);

            // Set user login and authenticate
            securityViewModel.Login = login;
            Assert.IsTrue(_testBll.AuthenticateUser("MyPassword"));

            Assert.IsTrue(securityViewModel.IsInRole("User"));
            Assert.IsTrue(securityViewModel.IsInRole("Guest"));
            Assert.IsFalse(securityViewModel.IsInRole("Admin"));

            // Drop the guest role
            _testBll.DropRoleFromUser("Guest");

            // Assert roles are correct
            Assert.IsTrue(securityViewModel.IsInRole("User"));
            Assert.IsFalse(securityViewModel.IsInRole("Guest"));
            Assert.IsFalse(securityViewModel.IsInRole("Admin"));

            _testBll.DropTable();
        }

        [TestMethod]
        public void UserLoginHasRegisteredRoles()
        {
            var securityViewModel = Container.Resolve<ISecurityViewModel>();

            securityViewModel.Login = "NewUser@MyDomain.com";
            const string password = "MyPassword"; // default password

            var status = _testBll.AuthenticateUser(password);
            Assert.IsFalse(status);

            var userRecord = new UserEntity
                                 {
                                     LastName = "User",
                                     FirstName = "New",
                                     Password = "MyPassword",
                                     PrimaryEmail = "NewUser@MyDomain.com",
                                 };
            var roles = new List<RoleEntity>
                            {
                                new RoleEntity { Role="User"}
                            };
            
            // Create the user with the role of User
            _testBll.CreateUsersWithRoles(userRecord, roles);

            // Set the security view model login
            securityViewModel.Login = userRecord.PrimaryEmail;

            // authenticate user - the security view model
            // will hold authentication results
            _testBll.AuthenticateUser("MyPassword");

            Assert.AreEqual("NewUser@MyDomain.com", securityViewModel.Login);
            Assert.IsTrue(securityViewModel.IsAuthenticated);

            // Only the User roles should be available
            Assert.IsTrue(securityViewModel.IsInRole("User"));
            Assert.IsFalse(securityViewModel.IsInRole("Admin"));
            Assert.IsFalse(securityViewModel.IsInRole("Guest"));

            var args = new DalEventArgs {PrimaryKey = securityViewModel.Id};
            
            // Read record for the specified primary key (above)
            // so that we can ensure it was created successfully
            Assert.AreEqual(StatusCode.Success, _testBll.Read(args));

            // Get the user results from the args 
            var createdUser = args.GetResults<UserEntity>();

            // Assert user record was created 
            Assert.IsNotNull(createdUser);
            Assert.AreEqual("New", createdUser.FirstName);
            Assert.AreEqual("User", createdUser.LastName);
            Assert.AreEqual("NewUser@MyDomain.com", createdUser.PrimaryEmail);
            Assert.AreEqual("MyPassword", createdUser.Password);

            _testBll.DropTable();
        }


        [TestMethod]
        public void EnsureInvalidLoginFails()
        {
            var securityViewModel = Container.Resolve<ISecurityViewModel>();
            securityViewModel.Login = "admin@mydomain.net";
            const string password = "Invalid password";

            var status = _testBll.AuthenticateUser(password);
            Assert.IsFalse(status);

            _testBll.DropTable();
        }


        [TestMethod]
        public void AdminLoginHasDefaultRoles()
        {
            var securityViewModel = Container.Resolve<ISecurityViewModel>();
            securityViewModel.Login = "admin@mydomain.net";
            const string password = "password"; // default password

            var status = _testBll.AuthenticateUser(password);
            Assert.IsTrue(status);
            Assert.IsTrue(securityViewModel.IsInRole("Admin"));
            Assert.IsTrue(securityViewModel.IsInRole("User"));
            Assert.IsFalse(securityViewModel.IsInRole("Guest"));

            _testBll.DropTable();
        }


        [TestMethod]
        public void EmptyUserInfoTableCreatesDefaults()
        {
            var resultArgs = new DalEventArgs();

            var status = _testBll.List(resultArgs);
            Assert.AreEqual(StatusCode.Success, status);

            var list = new List<UserEntity>(
                resultArgs.GetResults<IEnumerable<UserEntity>>());

            Assert.IsTrue(list.Count > 0, "Default UserInfo should have been created");

            // Cleanup
            _testBll.DropTable();
        }
    }
}
